@@ -145,7 +145,7 @@
          "$NumTrig" "$October" "$On" "$OnceFile" "$ParseUntriggered" "$Pm"
          "$PrefixLineNo" "$PSCal" "$RunOff" "$Saturday" "$September"
          "$SimpleCal" "$SortByDate" "$SortByPrio" "$SortByTime" "$SubsIndent"
-         "$Sunday" "$SuppressImplicitWarnings" "$SuppressLRM" "$SysInclude" "$T" "$Td"
+         "$Sunday" "$SuppressImplicitWarnings" "$SuppressLRM" "$SysInclude" "$T" "$Tb" "$Td"
          "$TerminalBackground" "$Thursday" "$TimeSep" "$TimetIs64bit" "$Tm" "$Today"
          "$Tomorrow" "$Tt" "$Tuesday" "$Tw" "$Ty" "$U" "$Ud" "$Um"
          "$UntimedFirst" "$Use256Colors" "$UseBGVTColors" "$UseTrueColors"
@@ -164,19 +164,19 @@
 (defconst remind-builtin-functions
   (sort
    (list "_" "abs" "access" "adawn" "adusk" "ampm" "ansicolor" "args" "asc"
-         "baseyr" "char" "choose" "coerce" "columns" "current" "date"
+         "baseyr" "catch" "catcherr" "char" "choose" "coerce" "columns" "const" "current" "date"
          "datepart" "datetime" "dawn" "day" "daysinmon" "defined" "dosubst"
-         "dusk" "easterdate" "escape" "evaltrig" "filedate" "filedatetime"
+         "dusk" "easterdate" "escape" "eval" "evaltrig" "filedate" "filedatetime"
          "filedir" "filename" "getenv" "hebdate" "hebday" "hebmon" "hebyear"
-         "hour" "htmlescape" "htmlstriptags" "iif" "index" "isany" "isdst"
+         "hour" "htmlescape" "htmlstriptags" "iif" "index" "isany" "isconst" "isdst"
          "isleap" "isomitted" "language" "localtoutc" "lower" "max" "min"
          "minsfromutc" "minute" "mon" "monnum" "moondate" "moondatetime"
          "moonphase" "moonrise" "moonrisedir" "moonset" "moonsetdir" "moontime"
-         "multitrig" "ndawn" "ndusk" "nonomitted" "now" "ord" "orthodoxeaster"
+         "multitrig" "ndawn" "ndusk" "nonconst" "nonomitted" "now" "ord" "orthodoxeaster"
          "ostype" "pad" "plural" "psmoon" "psshade" "realcurrent" "realnow"
          "realtoday" "rows" "sgn" "shell" "shellescape" "slide" "soleq"
          "stdout" "strlen" "substr" "sunrise" "sunset" "time" "timepart"
-         "timezone" "today" "trig" "trigback" "trigdate" "trigdatetime"
+         "timezone" "today" "trig" "trigback" "trigbase" "trigdate" "trigdatetime"
          "trigdelta" "trigduration" "trigeventduration" "trigeventstart"
          "trigfrom" "trigger" "triginfo" "trigpriority" "trigrep"
          "trigscanfrom" "trigtags" "trigtime" "trigtimedelta" "trigtimerep"
