@@ -55,7 +55,7 @@ typedef struct cal_entry {
     DynamicBuffer tags;
     char passthru[PASSTHRU_LEN+1];
     int duration;
-    char *filename;
+    char const *filename;
     int lineno;
     int lineno_start;
     Trigger trig;
@@ -1539,7 +1539,6 @@ static int WriteOneColLine(int col)
             CalColumn[col] = e->next;
             free(e->text);
             free(e->raw_text);
-            free(e->filename);
             if (e->wc_text) free(e->wc_text);
             FreeTrigInfoChain(e->infos);
             free(e);
@@ -1626,7 +1625,6 @@ static int WriteOneColLine(int col)
             CalColumn[col] = e->next;
             free(e->text);
             free(e->raw_text);
-            free(e->filename);
             if (e->wc_text) free(e->wc_text);
             FreeTrigInfoChain(e->infos);
             free(e);
@@ -1645,7 +1643,6 @@ static int WriteOneColLine(int col)
             PrintLeft("", ColSpaces, ' ');
             CalColumn[col] = e->next;
             free(e->text);
-            free(e->filename);
 #ifdef REM_USE_WCHAR
             if (e->wc_text) free(e->wc_text);
 #endif
@@ -1708,7 +1705,6 @@ static int WriteOneColLine(int col)
         if (!*s && !e->next) {
             CalColumn[col] = e->next;
             free(e->text);
-            free(e->filename);
 #ifdef REM_USE_WCHAR
             if (e->wc_text) free(e->wc_text);
 #endif
@@ -1757,12 +1753,13 @@ static void GenerateCalEntries(int col)
         s = FindInitialToken(&tok, CurLine);
 
         /* Should we ignore it? */
-        if (NumIfs &&
-            tok.type != T_If &&
+        if (tok.type != T_If &&
             tok.type != T_Else &&
             tok.type != T_EndIf &&
             tok.type != T_IfTrig &&
-            ShouldIgnoreLine())
+            tok.type != T_Set &&
+            tok.type != T_Fset &&
+            should_ignore_line())
         {
             /* DO NOTHING */
         }
@@ -2275,7 +2272,7 @@ static int DoCalRem(ParsePtr p, int col)
         }
         e->infos = NULL;
         e->nonconst_expr = nonconst_expr;
-        e->if_depth = NumIfs;
+        e->if_depth = get_if_pointer() - get_base_if_pointer();
         e->trig = trig;
         e->tt = tim;
 #ifdef REM_USE_WCHAR
@@ -2316,17 +2313,7 @@ static int DoCalRem(ParsePtr p, int col)
         FreeTrig(&trig);
         e->duration = tim.duration;
         e->priority = trig.priority;
-        e->filename = StrDup(FileName);
-        if(!e->filename) {
-            FreeTrigInfoChain(e->infos);
-            if (e->text) free(e->text);
-            if (e->raw_text) free(e->raw_text);
-#ifdef REM_USE_WCHAR
-            if (e->wc_text) free(e->wc_text);
-#endif
-            free(e);
-            return E_NO_MEM;
-        }
+        e->filename = GetCurrentFilename();
         e->lineno = LineNo;
         e->lineno_start = LineNoStart;
 
@@ -2463,6 +2450,10 @@ void WriteJSONTrigger(Trigger const *t, int include_tags, int today)
     if (t->rep) {
         PrintJSONKeyPairInt("rep", t->rep);
     }
+    if (t->d != NO_DAY && t->m != NO_MON && t->y != NO_YR) {
+        printf("\"trigbase\":\"%04d-%02d-%-2d\",",
+               t->y, t->m+1, t->d);
+    }
     /* Local omit is an array of days from 0=monday to 6=sunday.
        We convert to array of strings */
     if (t->localomit != NO_WD) {
@@ -2668,7 +2659,6 @@ static void WriteSimpleEntries(int col, int dse)
 
         free(e->text);
         free(e->raw_text);
-        free(e->filename);
         FreeTrigInfoChain(e->infos);
 #ifdef REM_USE_WCHAR
         if (e->wc_text) free(e->wc_text);
