@@ -20,6 +20,7 @@
 #include <errno.h>
 #include <ctype.h>
 #include <sys/stat.h>
+#include <stddef.h>
 
 #ifdef TM_IN_SYS_TIME
 #include <sys/time.h>
@@ -79,18 +80,27 @@ typedef struct {
     FilenameChain *chain;
     int LineNo;
     int LineNoStart;
-    unsigned int IfFlags;
-    int NumIfs;
-    int IfLinenos[IF_NEST];
+    int base_if_pointer;
     long offset;
     CachedLine *CLine;
     int ownedByMe;
 } IncludeStruct;
 
+typedef struct fn_entry {
+    struct hash_link link;
+    char const *fname;
+} FilenameHashEntry;
+
+/* A hash table to hold unique copies of all the filenames we process */
+static hash_table FilenameHashTable;
+
 static CachedFile *CachedFiles = (CachedFile *) NULL;
 static CachedLine *CLine = (CachedLine *) NULL;
 static DirectoryFilenameChain *CachedDirectoryChains = NULL;
 
+/* Current filename */
+static char const *FileName = NULL;
+
 static FILE *fp;
 
 static IncludeStruct IStack[INCLUDE_NEST];
@@ -104,6 +114,64 @@ static int CheckSafetyAux (struct stat *statbuf);
 static int PopFile (void);
 static int IncludeCmd(char const *);
 
+static unsigned int FnHashFunc(void *x)
+{
+    FilenameHashEntry *e = (FilenameHashEntry *) x;
+    return HashVal_preservecase(e->fname);
+}
+
+static int FnCompareFunc(void *a, void *b)
+{
+    FilenameHashEntry *e1 = (FilenameHashEntry *) a;
+    FilenameHashEntry *e2 = (FilenameHashEntry *) b;
+    return strcmp(e1->fname, e2->fname);
+}
+
+void InitFiles(void)
+{
+    if (hash_table_init(&FilenameHashTable, offsetof(FilenameHashEntry, link),
+                        FnHashFunc, FnCompareFunc) < 0) {
+        fprintf(ErrFp, "Unable to initialize filename hash table: Out of memory.  Exiting.\n");
+        exit(1);
+    }
+}
+
+void SetCurrentFilename(char const *fname)
+{
+    FilenameHashEntry *e;
+    FilenameHashEntry candidate;
+    candidate.fname = fname;
+
+    e = (FilenameHashEntry *) hash_table_find(&FilenameHashTable, &candidate);
+    if (!e) {
+        e = NEW(FilenameHashEntry);
+        if (!e) {
+            fprintf(ErrFp, "Out of Memory!\n");
+            exit(1);
+        }
+        e->fname = strdup(fname);
+        if (!e->fname) {
+            fprintf(ErrFp, "Out of Memory!\n");
+            exit(1);
+        }
+        hash_table_insert(&FilenameHashTable, e);
+    }
+    FileName = e->fname;
+}
+
+char const *GetCurrentFilename(void)
+{
+    if (FileName) {
+        if (!strcmp(FileName, "-")) {
+            return "-stdin-";
+        } else {
+            return FileName;
+        }
+    } else {
+        return "";
+    }
+}
+
 static void
 got_a_fresh_line(void)
 {
@@ -334,7 +402,7 @@ int OpenFile(char const *fname)
                 fprintf(ErrFp, "\n");
             }
             CLine = h->cache;
-            STRSET(FileName, fname);
+            SetCurrentFilename(fname);
             LineNo = 0;
             LineNoStart = 0;
             if (!h->ownedByMe) {
@@ -389,7 +457,7 @@ int OpenFile(char const *fname)
             }
         }
     }
-    STRSET(FileName, fname);
+    SetCurrentFilename(fname);
     LineNo = 0;
     LineNoStart = 0;
     if (FileName) return OK; else return E_NO_MEM;
@@ -546,15 +614,9 @@ static int NextChainedFile(IncludeStruct *i)
 static int PopFile(void)
 {
     IncludeStruct *i;
-    int j;
 
-    if (!Hush && NumIfs) {
-        Eprint("%s", GetErr(E_MISS_ENDIF));
-        for (j=NumIfs-1; j >=0; j--) {
-            fprintf(ErrFp, tr("%s(%d): IF without ENDIF"), FileName, IfLinenos[j]);
-            fprintf(ErrFp, "\n");
-        }
-    }
+    pop_excess_ifs(FileName);
+
     if (!IStackPtr) return E_EOF;
     i = &IStack[IStackPtr-1];
 
@@ -574,12 +636,10 @@ static int PopFile(void)
 
     LineNo = i->LineNo;
     LineNoStart = i->LineNoStart;
-    IfFlags = i->IfFlags;
-    memcpy(IfLinenos, i->IfLinenos, IF_NEST);
-    NumIfs = i->NumIfs;
+    set_base_if_pointer(i->base_if_pointer);
     CLine = i->CLine;
     fp = NULL;
-    STRSET(FileName, i->filename);
+    SetCurrentFilename(i->filename);
     if (!i->ownedByMe) {
         RunDisabled |= RUN_NOTOWNER;
     } else {
@@ -599,7 +659,6 @@ static int PopFile(void)
         if (fp != stdin)
             (void) fseek(fp, i->offset, 0);  /* Trust that it works... */
     }
-    free((char *) i->filename);
     return OK;
 }
 
@@ -904,21 +963,11 @@ static int IncludeCmd(char const *cmd)
     }
     fname = DBufValue(&buf);
 
-    if (FileName) {
-        i->filename = StrDup(FileName);
-        if (!i->filename) {
-            DBufFree(&buf);
-            return E_NO_MEM;
-        }
-    } else {
-        i->filename = NULL;
-    }
+    i->filename = FileName;
     i->ownedByMe = 1;
     i->LineNo = LineNo;
     i->LineNoStart = LineNo;
-    i->NumIfs = NumIfs;
-    i->IfFlags = IfFlags;
-    memcpy(i->IfLinenos, IfLinenos, IF_NEST);
+    i->base_if_pointer = get_base_if_pointer();
     i->CLine = CLine;
     i->offset = -1L;
     i->chain = NULL;
@@ -927,8 +976,8 @@ static int IncludeCmd(char const *cmd)
         FCLOSE(fp);
     }
     IStackPtr++;
-    NumIfs = 0;
-    IfFlags = 0;
+
+    set_base_if_pointer(get_if_pointer());
 
     /* If the file is cached, use it */
     h = CachedFiles;
@@ -939,7 +988,7 @@ static int IncludeCmd(char const *cmd)
                 fprintf(ErrFp, "\n");
             }
             CLine = h->cache;
-            STRSET(FileName, fname);
+            SetCurrentFilename(fname);
             DBufFree(&buf);
             LineNo = 0;
             LineNoStart = 0;
@@ -991,7 +1040,7 @@ static int IncludeCmd(char const *cmd)
         CLine = CachedFiles->cache;
         LineNo = 0;
         LineNoStart = 0;
-        STRSET(FileName, fname);
+        SetCurrentFilename(fname);
         DBufFree(&buf);
         return OK;
     }
@@ -1020,17 +1069,10 @@ int IncludeFile(char const *fname)
     if (IStackPtr >= INCLUDE_NEST) return E_NESTED_INCLUDE;
     i = &IStack[IStackPtr];
 
-    if (FileName) {
-        i->filename = StrDup(FileName);
-        if (!i->filename) return E_NO_MEM;
-    } else {
-        i->filename = NULL;
-    }
+    i->filename = FileName;
     i->LineNo = LineNo;
     i->LineNoStart = LineNoStart;
-    i->NumIfs = NumIfs;
-    i->IfFlags = IfFlags;
-    memcpy(i->IfLinenos, IfLinenos, IF_NEST);
+    i->base_if_pointer = get_base_if_pointer();
     i->CLine = CLine;
     i->offset = -1L;
     i->chain = NULL;
@@ -1045,8 +1087,7 @@ int IncludeFile(char const *fname)
     }
 
     IStackPtr++;
-    NumIfs = 0;
-    IfFlags = 0;
+    set_base_if_pointer(get_if_pointer());
 
 #ifdef HAVE_GLOB
     /* If it's a directory, set up the glob chain here. */
