@@ -113,14 +113,10 @@ EXTERN  INIT(   int     SynthesizeTags, 0);
 EXTERN  INIT(   int     ScFormat, SC_AMPM);
 EXTERN  INIT(   int     MaxSatIter, 1000);
 EXTERN  INIT(   int     MaxStringLen, MAX_STR_LEN);
-EXTERN  INIT(   char    *FileName, NULL);
 EXTERN  INIT(   int     UseStdin, 0);
 EXTERN  INIT(   int     PurgeMode, 0);
 EXTERN  INIT(   int     PurgeIncludeDepth, 0);
 EXTERN  INIT(   FILE    *PurgeFP,  NULL);
-EXTERN  INIT(   int     NumIfs,    0);
-EXTERN  INIT(   unsigned int IfFlags,   0);
-EXTERN  INIT(   int     IfLinenos[IF_NEST], {0});
 EXTERN  INIT(   int     LastTrigValid, 0);
 EXTERN  Trigger  LastTrigger;
 EXTERN  TimeTrig LastTimeTrig;
@@ -143,6 +139,9 @@ EXTERN  INIT(   int     UseTrueColors, 0);
 EXTERN  INIT(   int     TerminalBackground, TERMINAL_BACKGROUND_UNKNOWN);
 EXTERN  INIT(   int     DedupeReminders, 0);
 
+/* Suppress ALL error output during a catch() */
+EXTERN  INIT(   int     SuppressErrorOutputInCatch, 0);
+
 /* Latitude and longitude */
 EXTERN  INIT(   int       LatDeg, 0);
 EXTERN  INIT(   int       LatMin, 0);
@@ -244,3 +243,6 @@ EXTERN int SuppressLRM
 
 /* Translatable messages */
 extern char const *translatables[];
+
+/* Array for "folding" years to similar years */
+extern int FoldArray[2][7];
