@@ -186,6 +186,7 @@ void InitRemind(int argc, char const *argv[])
     InitUserFunctions();
 
     InitTranslationTable();
+    InitFiles();
 
     /* If stdout is a terminal, initialize $FormWidth to terminal width-8,
        but clamp to [20, 500] */
@@ -635,7 +636,7 @@ void InitRemind(int argc, char const *argv[])
                 while (*arg) {
                     switch(*arg++) {
                     case 's': case 'S': DebugFlag |= DB_PARSE_EXPR;  break;
-                    case 'h': case 'H': DebugFlag |= DB_HASHSTATS;  break;
+                    case 'h': case 'H': DebugFlag |= DB_HASHSTATS;   break;
                     case 'e': case 'E': DebugFlag |= DB_ECHO_LINE;   break;
                     case 'x': case 'X': DebugFlag |= DB_PRTEXPR;     break;
                     case 't': case 'T': DebugFlag |= DB_PRTTRIG;     break;
@@ -643,6 +644,8 @@ void InitRemind(int argc, char const *argv[])
                     case 'l': case 'L': DebugFlag |= DB_PRTLINE;     break;
                     case 'f': case 'F': DebugFlag |= DB_TRACE_FILES; break;
                     case 'q': case 'Q': DebugFlag |= DB_TRANSLATE;   break;
+                    case 'n': case 'N': DebugFlag |= DB_NONCONST;    break;
+                    case 'u': case 'U': DebugFlag |= DB_UNUSED_VARS; break;
                     default:
                         fprintf(ErrFp, GetErr(M_BAD_DB_FLAG), *(arg-1));
                         fprintf(ErrFp, "\n");
@@ -835,7 +838,7 @@ void Usage(void)
     fprintf(ErrFp, " -q     Don't queue timed reminders\n");
     fprintf(ErrFp, " -f     Trigger timed reminders by staying in foreground\n");
     fprintf(ErrFp, " -z[n]  Enter daemon mode, waking every n (1) minutes.\n");
-    fprintf(ErrFp, " -d...  Debug: e=echo x=expr-eval t=trig v=dumpvars l=showline f=tracefiles\n");
+    fprintf(ErrFp, " -d...  Debug: See man page for details\n");
     fprintf(ErrFp, " -e     Divert messages normally sent to stderr to stdout\n");
     fprintf(ErrFp, " -b[n]  Time format for cal: 0=am/pm, 1=24hr, 2=none\n");
     fprintf(ErrFp, " -x[n]  Iteration limit for SATISFY clause (def=1000)\n");
@@ -1003,7 +1006,7 @@ static void InitializeVar(char const *str)
         }
         val.type = INT_TYPE;
         val.v.val = 0;
-        r = SetVar(varname, &val);
+        r = SetVar(varname, &val, 1);
         if (!r) {
             r = PreserveVar(varname);
         }
@@ -1043,7 +1046,7 @@ static void InitializeVar(char const *str)
         return;
     }
 
-    r=SetVar(varname, &val);
+    r=SetVar(varname, &val, 1);
     if (r) {
         fprintf(ErrFp, GetErr(M_I_OPTION), GetErr(r));
         fprintf(ErrFp, "\n");
