@@ -60,6 +60,9 @@ exitfunc(void)
     /* Kill any execution-time-limiter process */
     unlimit_execution_time();
 
+    if (DebugFlag & DB_UNUSED_VARS) {
+        DumpUnusedVars();
+    }
     if (DebugFlag & DB_HASHSTATS) {
         fflush(stdout);
         fflush(ErrFp);
@@ -169,7 +172,7 @@ int main(int argc, char *argv[])
         DoReminders();
 
         if (DebugFlag & DB_DUMP_VARS) {
-            DumpVarTable();
+            DumpVarTable(0);
             DumpSysVarByName(NULL);
         }
 
@@ -283,12 +286,13 @@ static void DoReminders(void)
         s = FindInitialToken(&tok, CurLine);
 
         /* Should we ignore it? */
-        if (NumIfs &&
-            tok.type != T_If &&
+        if (tok.type != T_If &&
             tok.type != T_Else &&
             tok.type != T_EndIf &&
             tok.type != T_IfTrig &&
-            ShouldIgnoreLine())
+            tok.type != T_Set &&
+            tok.type != T_Fset &&
+            should_ignore_line())
         {
             /*** IGNORE THE LINE ***/
             if (PurgeMode) {
@@ -393,7 +397,9 @@ static void DoReminders(void)
                     PurgeEchoLine("%s\n", CurLine);
                 } else {
                     if (r) {
-                        PurgeEchoLine("#!P! Could not parse next line: %s\n", GetErr(r));
+                        if (!Hush) {
+                            PurgeEchoLine("#!P! Could not parse next line: %s\n", GetErr(r));
+                        }
                         PurgeEchoLine("%s\n", CurLine);
                     }
                 }
@@ -896,21 +902,17 @@ void Wprint(char const *fmt, ...)
 {
     va_list argptr;
 
+    char const *fname = GetCurrentFilename();
+    if (SuppressErrorOutputInCatch) {
+        return;
+    }
 
     /* We can't use line_range because caller might have used it */
-    if (FileName) {
-        if (strcmp(FileName, "-")) {
-            if (LineNoStart == LineNo) {
-                (void) fprintf(ErrFp, "%s(%d): ", FileName, LineNo);
-            } else {
-                (void) fprintf(ErrFp, "%s(%d:%d): ", FileName, LineNoStart, LineNo);
-            }
+    if (fname) {
+        if (LineNoStart == LineNo) {
+            (void) fprintf(ErrFp, "%s(%d): ", fname, LineNo);
         } else {
-            if (LineNoStart == LineNo) {
-                (void) fprintf(ErrFp, "-stdin-(%d): ", LineNo);
-            } else {
-                (void) fprintf(ErrFp, "-stdin-(%d:%d): ", LineNoStart, LineNo);
-            }
+            (void) fprintf(ErrFp, "%s(%d:%d): ", fname, LineNoStart, LineNo);
         }
     }
 
@@ -928,20 +930,27 @@ void Wprint(char const *fmt, ...)
 void Eprint(char const *fmt, ...)
 {
     va_list argptr;
-    char const *fname;
 
-    /* Check if more than one error msg. from this line */
-    if (!FreshLine && !ShowAllErrors) return;
-
-    if (!FileName) {
+    if (SuppressErrorOutputInCatch) {
         return;
     }
 
-    if (strcmp(FileName, "-")) {
-        fname = FileName;
-    } else {
-        fname = "-stdin-";
+    if (should_ignore_line()) {
+        return;
     }
+
+    char const *fname = GetCurrentFilename();
+    if (!fname) {
+        return;
+    }
+
+    if (LineNo < 1) {
+        /* Not yet processing a file */
+        return;
+    }
+    /* Check if more than one error msg. from this line */
+    if (!FreshLine && !ShowAllErrors) return;
+
     if (FreshLine) {
         /* We can't use line_range because caller might have used it */
         if (LineNo == LineNoStart) {
@@ -1122,32 +1131,29 @@ int DoIf(ParsePtr p)
 {
     Value v;
     int r;
-    unsigned syndrome;
 
-    if ((size_t) NumIfs >= IF_NEST) return E_NESTED_IF;
+    if (if_stack_full()) {
+        return E_NESTED_IF;
+    }
 
-    if (ShouldIgnoreLine()) syndrome = IF_TRUE | BEFORE_ELSE;
-    else {
+    if (should_ignore_line()) {
+        push_if(1, 1);
+        return OK;
+    } else {
         if ( (r = EvaluateExpr(p, &v)) ) {
-            syndrome = IF_TRUE | BEFORE_ELSE;
             Eprint("%s", GetErr(r));
+            push_if(1, 0);
         } else
             if (truthy(&v)) {
-                syndrome = IF_TRUE | BEFORE_ELSE;
+                push_if(1, !p->nonconst_expr);
             } else {
-                syndrome = IF_FALSE | BEFORE_ELSE;
-                if (PurgeMode) {
+                push_if(0, !p->nonconst_expr);
+                if (PurgeMode && !Hush) {
                     PurgeEchoLine("%s\n", "#!P: The next IF evaluated false...");
                     PurgeEchoLine("%s\n", "#!P: REM statements in IF block not checked for purging.");
                 }
             }
     }
-
-    IfLinenos[NumIfs] = LineNo;
-    NumIfs++;
-    IfFlags &= ~(IF_MASK << (2*NumIfs - 2));
-    IfFlags |= syndrome << (2 * NumIfs - 2);
-    if (ShouldIgnoreLine()) return OK;
     return VerifyEoln(p);
 }
 
@@ -1159,21 +1165,16 @@ int DoIf(ParsePtr p)
 /***************************************************************/
 int DoElse(ParsePtr p)
 {
-    unsigned syndrome;
+    int was_ignoring = should_ignore_line();
 
-    int was_ignoring = ShouldIgnoreLine();
-
-    if (!NumIfs) return E_ELSE_NO_IF;
-
-    syndrome = IfFlags >> (2 * NumIfs - 2);
-
-    if ((syndrome & IF_ELSE_MASK) == AFTER_ELSE) return E_ELSE_NO_IF;
-
-    IfFlags |= AFTER_ELSE << (2 * NumIfs - 2);
-    if (PurgeMode && ShouldIgnoreLine() && !was_ignoring) {
+    int r = encounter_else();
+    if (PurgeMode && should_ignore_line() && !was_ignoring && !Hush) {
         PurgeEchoLine("%s\n", "#!P: The previous IF evaluated true.");
         PurgeEchoLine("%s\n", "#!P: REM statements in ELSE block not checked for purging");
     }
+    if (r != OK) {
+        return r;
+    }
     return VerifyEoln(p);
 }
 
@@ -1184,8 +1185,10 @@ int DoElse(ParsePtr p)
 /***************************************************************/
 int DoEndif(ParsePtr p)
 {
-    if (!NumIfs) return E_ENDIF_NO_IF;
-    NumIfs--;
+    int r = encounter_endif();
+    if (r != OK) {
+        return r;
+    }
     return VerifyEoln(p);
 }
 
@@ -1199,15 +1202,19 @@ int DoEndif(ParsePtr p)
 int DoIfTrig(ParsePtr p)
 {
     int r, err;
-    unsigned syndrome;
     Trigger trig;
     TimeTrig tim;
     int dse;
 
 
-    if ((size_t) NumIfs >= IF_NEST) return E_NESTED_IF;
-    if (ShouldIgnoreLine()) syndrome = IF_TRUE | BEFORE_ELSE;
-    else {
+    if (if_stack_full()) {
+        return E_NESTED_IF;
+    }
+
+    if (should_ignore_line()) {
+        push_if(1, 0);
+        return OK;
+    } else {
         if ( (r=ParseRem(p, &trig, &tim)) ) return r;
         if (trig.typ != NO_TYPE) return E_PARSE_ERR;
         dse = ComputeTrigger(trig.scanfrom, &trig, &tim, &r, 1);
@@ -1217,49 +1224,24 @@ int DoIfTrig(ParsePtr p)
                     Eprint("%s", GetErr(r));
                 }
             }
-            syndrome = IF_FALSE | BEFORE_ELSE;
-        }
-        else {
+            push_if(0, 0);
+        } else {
             if (ShouldTriggerReminder(&trig, &tim, dse, &err)) {
-                syndrome = IF_TRUE | BEFORE_ELSE;
+                push_if(1, 0);
             } else {
-                syndrome = IF_FALSE | BEFORE_ELSE;
+                push_if(0, 0);
+                if (PurgeMode && !Hush) {
+                    PurgeEchoLine("%s\n", "#!P: The next IFTRIG did not trigger.");
+                    PurgeEchoLine("%s\n", "#!P: REM statements in IFTRIG block not checked for purging.");
+                }
             }
         }
-        if (syndrome == (IF_FALSE | BEFORE_ELSE) && PurgeMode) {
-            PurgeEchoLine("%s\n", "#!P: The next IFTRIG did not trigger.");
-            PurgeEchoLine("%s\n", "#!P: REM statements in IFTRIG block not checked for purging.");
-        }
         FreeTrig(&trig);
     }
-    NumIfs++;
-    IfFlags &= ~(IF_MASK << (2*NumIfs - 2));
-    IfFlags |= syndrome << (2 * NumIfs - 2);
     return OK;
 }
 
 
-/***************************************************************/
-/*                                                             */
-/*  ShouldIgnoreLine - given the current state of the IF       */
-/*  stack, should we ignore the current line?                  */
-/*                                                             */
-/***************************************************************/
-int ShouldIgnoreLine(void)
-{
-    register int i, syndrome;
-
-/* Algorithm - go from outer to inner, and if any should be ignored, then
-   ignore the whole. */
-
-    for (i=0; i<NumIfs; i++) {
-        syndrome = (IfFlags >> (i*2)) & IF_MASK;
-        if (syndrome == IF_TRUE+AFTER_ELSE ||
-            syndrome == IF_FALSE+BEFORE_ELSE) return 1;
-    }
-    return 0;
-}
-
 /***************************************************************/
 /*                                                             */
 /*  VerifyEoln                                                 */
@@ -1373,6 +1355,17 @@ int DoDebug(ParsePtr p)
             if (val) DebugFlag |= DB_TRACE_FILES;
             else     DebugFlag &= ~DB_TRACE_FILES;
             break;
+
+        case 'n':
+        case 'N':
+            if (val) DebugFlag |= DB_NONCONST;
+            else     DebugFlag &= ~DB_NONCONST;
+            break;
+        case 'u':
+        case 'U':
+            if (val) DebugFlag |= DB_UNUSED_VARS;
+            else     DebugFlag &= ~DB_UNUSED_VARS;
+            break;
         default:
             Wprint(GetErr(M_BAD_DB_FLAG), ch);
             break;
@@ -1560,10 +1553,10 @@ int DoErrMsg(ParsePtr p)
    non-leap year beginning on Wednesday, and FoldArray[1][5] is a
    leap year beginning on Saturday.  Used to fold back dates which
    are too high for the standard Unix representation.
-   NOTE:  This implies that you cannot set BASE > 2001!!!!! */
-static int FoldArray[2][7] = {
-    {2001, 2002, 2003, 2009, 2010, 2005, 2006},
-    {2024, 2008, 2020, 2004, 2016, 2000, 2012}
+   NOTE:  This implies that you cannot set BASE > 2012!!!!! */
+int FoldArray[2][7] = {
+    {2035, 2030, 2031, 2026, 2027, 2033, 2034},
+    {2024, 2036, 2020, 2032, 2016, 2028, 2012}
 };
 
 int CalcMinsFromUTC(int dse, int tim, int *mins, int *isdst)
@@ -1581,7 +1574,7 @@ int CalcMinsFromUTC(int dse, int tim, int *mins, int *isdst)
 /* If the year is greater than 2037, some Unix machines have problems.
    Fold it back to a "similar" year and trust that the UTC calculations
    are still valid... */
-    if (FoldYear && yr>2037) {
+    if (yr > 2037 && (FoldYear || (sizeof(time_t) < (64/CHAR_BIT)))) {
         dse = DSE(yr, 0, 1);
         yr = FoldArray[IsLeapYear(yr)][dse%7];
     }
@@ -2131,6 +2124,14 @@ int GetOnceDate(void)
     return OnceDate;
 }
 
+char const *GetEnglishErr(int r)
+{
+    if (r < 0 || r >= NumErrs) {
+        r = E_SWERR;
+    }
+    return ErrMsg[r];
+}
+
 char const *GetErr(int r)
 {
     char const *msg;
