@@ -151,12 +151,13 @@ void strtolower(char *s);
 Var *FindVar (char const *str, int create);
 SysVar *FindSysVar (char const *name);
 int DeleteVar (char const *str);
-int SetVar (char const *str, Value const *val);
+int SetVar (char const *str, Value const *val, int nonconst_expr);
 int GetVarValue (char const *str, Value *val);
 int DoSet  (Parser *p);
 int DoUnset  (Parser *p);
 int DoDump (ParsePtr p);
-void DumpVarTable (void);
+void DumpVarTable (int dump_constness);
+void DumpUnusedVars(void);
 void DestroyVars (int all);
 int PreserveVar (char const *name);
 int DoPreserve  (Parser *p);
@@ -268,9 +269,11 @@ void InitDedupeTable(void);
 void InitVars(void);
 void InitUserFunctions(void);
 void InitTranslationTable(void);
+void InitFiles(void);
 char const *GetTranslatedString(char const *orig);
 int GetTranslatedStringTryingVariants(char const *orig, DynamicBuffer *out);
 char const *GetErr(int r);
+char const *GetEnglishErr(int r);
 char const *tr(char const *s);
 void print_escaped_string(FILE *fp, char const *s);
 void print_escaped_string_helper(FILE *fp, char const *s, int esc_for_remind, int json);
@@ -289,3 +292,19 @@ int GetMoonrise(int dse);
 int GetMoonset(int dse);
 int GetMoonrise_angle(int dse);
 int GetMoonset_angle(int dse);
+#define nonconst_debug(nc, ...) do { if ((DebugFlag & DB_NONCONST) && !nc) { Wprint(__VA_ARGS__); } } while(0)
+
+/* if-else handling */
+int push_if(int is_true, int was_constant);
+int if_stack_full(void);
+int encounter_else(void);
+int encounter_endif(void);
+int get_base_if_pointer(void);
+int get_if_pointer(void);
+void set_base_if_pointer(int n);
+int should_ignore_line(void);
+int in_constant_context(void);
+void pop_excess_ifs(char const *fname);
+
+void SetCurrentFilename(char const *fname);
+char const *GetCurrentFilename(void);
