@@ -66,6 +66,7 @@ typedef struct {
     int nargs;
     Value *args;
     Value retval;
+    int nonconst;
 } func_info;
 
 /* Forward reference */
@@ -104,6 +105,10 @@ typedef struct var {
     struct hash_link link;
     char name[VAR_NAME_LEN+1];
     char preserve;
+    char is_constant;
+    char used_since_set;
+    char const *filename;
+    int lineno;
     Value v;
 } Var;
 
@@ -215,6 +220,8 @@ typedef Parser *ParsePtr;  /* Pointer to parser structure */
 #define DB_PARSE_EXPR   0x040
 #define DB_HASHSTATS    0x080
 #define DB_TRANSLATE    0x100
+#define DB_NONCONST     0x200
+#define DB_UNUSED_VARS  0x400
 
 /* Enumeration of the tokens */
 enum TokTypes
@@ -238,15 +245,6 @@ typedef struct {
     int val;
 } Token;
 
-/* Flags for the state of the "if" stack */
-#define IF_TRUE      0x00
-#define IF_FALSE     0x01
-#define BEFORE_ELSE  0x00
-#define AFTER_ELSE   0x02
-#define IF_MASK      0x03
-#define IF_TRUE_MASK 0x01
-#define IF_ELSE_MASK 0x02
-
 /* Flags for the DoSubst function */
 #define NORMAL_MODE  0
 #define CAL_MODE     1
@@ -258,6 +256,7 @@ typedef struct {
 #define RUN_CMDLINE  0x01
 #define RUN_SCRIPT   0x02
 #define RUN_NOTOWNER 0x04
+#define RUN_IN_EVAL  0x08
 
 /* Flags for the SimpleCalendar format */
 #define SC_AMPM   0   /* Time shown as 3:00am, etc. */
@@ -303,6 +302,7 @@ typedef struct {
 typedef struct udf_struct {
     struct hash_link link;
     char name[VAR_NAME_LEN+1];
+    char is_constant;
     expr_node *node;
     char **args;
     int nargs;
