@@ -181,12 +181,34 @@ int DoFset(ParsePtr p)
     Var *locals = NULL;
     Var local_array[MAX_FUNC_ARGS];
     int orig_namelen;
+    int suppress_redefined_function_warning = 0;
+    int ch;
 
     DynamicBuffer buf;
     DBufInit(&buf);
 
+
+    ch = ParseNonSpaceChar(p, &r, 1);
+    if (r) {
+        return r;
+    }
+    if (ch == '-') {
+        r = ParseToken(p, &buf);
+        if (r) {
+            return r;
+        }
+        if (strcmp(DBufValue(&buf), "-")) {
+            DBufFree(&buf);
+            return E_PARSE_ERR;
+        }
+        suppress_redefined_function_warning = 1;
+        DBufFree(&buf);
+    }
+
     /* Get the function name */
-    if ( (r=ParseIdentifier(p, &buf)) ) return r;
+    if ( (r=ParseIdentifier(p, &buf)) ) {
+        return r;
+    }
     if (*DBufValue(&buf) == '$') {
         DBufFree(&buf);
         return E_BAD_ID;
@@ -196,11 +218,25 @@ int DoFset(ParsePtr p)
     /* Convert to lower-case */
     strtolower(DBufValue(&buf));
 
+    /* If we're ignoring the line, just update is_constant flag if needed */
+    if (should_ignore_line()) {
+        if (in_constant_context()) {
+            DBufFree(&buf);
+            return OK;
+        }
+        existing = FindUserFunc(DBufValue(&buf));
+        if (existing) {
+            nonconst_debug(!existing->is_constant, tr("Potential function definition considered non-constant because of context"));
+            existing->is_constant = 0;
+        }
+        DBufFree(&buf);
+        return OK;
+    }
     /* If the function exists and was defined at the same line of the same
        file, do nothing */
     existing = FindUserFunc(DBufValue(&buf));
     if (existing) {
-        if (!strcmp(existing->filename, FileName) &&
+        if (!strcmp(existing->filename, GetCurrentFilename()) &&
             strcmp(existing->filename, "[cmdline]") &&
             existing->lineno == LineNo) {
             DBufFree(&buf);
@@ -208,8 +244,10 @@ int DoFset(ParsePtr p)
             return OK;
         }
         /* Warn about redefinition */
-        Wprint(tr("Function `%s' redefined: previously defined at %s(%s)"),
-               existing->name, existing->filename, line_range(existing->lineno_start, existing->lineno));
+        if (!suppress_redefined_function_warning) {
+            Wprint(tr("Function `%s' redefined: previously defined at %s(%s)"),
+                   existing->name, existing->filename, line_range(existing->lineno_start, existing->lineno));
+        }
     }
 
     /* Should be followed by '(' */
@@ -228,18 +266,20 @@ int DoFset(ParsePtr p)
         DBufFree(&buf);
         return E_NO_MEM;
     }
-    if (FileName) {
-        func->filename = StrDup(FileName);
+    if (GetCurrentFilename()) {
+        func->filename = GetCurrentFilename();
     } else {
-        func->filename = StrDup("[cmdline]");
-    }
-    if (!func->filename) {
-        free(func);
-        return E_NO_MEM;
+        func->filename = "[cmdline]";
     }
     func->lineno = LineNo;
     func->lineno_start = LineNoStart;
     func->recurse_flag = 0;
+    if (in_constant_context()) {
+        func->is_constant = 1;
+    } else {
+        nonconst_debug(0, tr("Function definition considered non-constant because of context"));
+        func->is_constant = 0;
+    }
     StrnCpy(func->name, DBufValue(&buf), VAR_NAME_LEN);
     DBufFree(&buf);
     if (!Hush) {
@@ -373,9 +413,6 @@ static void DestroyUserFunc(UserFunc *f)
     /* Free the function definition */
     if (f->node) free_expr_tree(f->node);
 
-    /* Free the filename */
-    if (f->filename) free( (char *) f->filename);
-
     /* Free arg names */
     if (f->args) {
         for (i=0; i<f->nargs; i++) {
