@@ -254,6 +254,35 @@ static int trig_date_func(int do_set, Value *val)
     }
     return OK;
 }
+
+static int trig_base_func(int do_set, Value *val)
+{
+    UNUSED(do_set);
+    if (LastTrigger.d != NO_DAY &&
+        LastTrigger.m != NO_MON &&
+        LastTrigger.y != NO_YR) {
+        val->type = DATE_TYPE;
+        val->v.val = DSE(LastTrigger.y, LastTrigger.m, LastTrigger.d);
+    } else {
+        val->type = INT_TYPE;
+        val->v.val = 0;
+    }
+    return OK;
+}
+
+static int trig_until_func(int do_set, Value *val)
+{
+    UNUSED(do_set);
+    if (LastTrigger.until == NO_UNTIL) {
+        val->type = INT_TYPE;
+        val->v.val = -1;
+    } else {
+        val->type = DATE_TYPE;
+        val->v.val = LastTrigger.until;
+    }
+    return OK;
+}
+
 static int trig_day_func(int do_set, Value *val)
 {
     int y, m, d;
@@ -531,6 +560,9 @@ Var *FindVar(char const *str, int create)
     v->v.type = INT_TYPE;
     v->v.v.val = 0;
     v->preserve = 0;
+    v->is_constant = 1;
+    v->filename = "";
+    v->lineno = 0;
     StrnCpy(v->name, str, VAR_NAME_LEN);
 
     hash_table_insert(&VHashTbl, v);
@@ -550,6 +582,9 @@ int DeleteVar(char const *str)
 
     v = FindVar(str, 0);
     if (!v) return E_NOSUCH_VAR;
+    if ((DebugFlag & DB_UNUSED_VARS) && !v->used_since_set) {
+        Eprint(tr("`%s' UNSET without being used (previous SET: %s:%d)"), str, v->filename, v->lineno);
+    }
     DestroyValue(v->v);
     hash_table_delete(&VHashTbl, v);
     return OK;
@@ -562,14 +597,28 @@ int DeleteVar(char const *str)
 /*  Set the indicated variable to the specified value.         */
 /*                                                             */
 /***************************************************************/
-int SetVar(char const *str, Value const *val)
+int SetVar(char const *str, Value const *val, int nonconst_expr)
 {
-    Var *v = FindVar(str, 1);
+    Var *v = NULL;
 
+    if (DebugFlag & DB_UNUSED_VARS) {
+        v = FindVar(str, 0);
+        if (v && !(v->used_since_set)) {
+            Eprint(tr("`%s' re-SET without being used (previous SET: %s:%d)"), str, v->filename, v->lineno);
+        }
+    }
+
+    if (!v) {
+        v = FindVar(str, 1);
+    }
     if (!v) return E_NO_MEM;  /* Only way FindVar can fail */
 
     DestroyValue(v->v);
     v->v = *val;
+    v->is_constant = ! nonconst_expr;
+    v->used_since_set = 0;
+    v->filename = GetCurrentFilename();
+    v->lineno = LineNo;
     return OK;
 }
 
@@ -590,6 +639,7 @@ int GetVarValue(char const *str, Value *val)
         Eprint("%s: `%s'", GetErr(E_NOSUCH_VAR), str);
         return E_NOSUCH_VAR;
     }
+    v->used_since_set = 1;
     return CopyValue(val, &v->v);
 }
 
@@ -603,13 +653,36 @@ int DoSet (Parser *p)
     Value v;
     int r;
     int ch;
+    int ignoring = should_ignore_line();
     DynamicBuffer buf;
     DynamicBuffer buf2;
     DBufInit(&buf);
     DBufInit(&buf2);
+    Var *var;
 
     r = ParseIdentifier(p, &buf);
-    if (r) return r;
+    if (r) {
+        DBufFree(&buf);
+        if (ignoring) {
+            return OK;
+        }
+        return r;
+    }
+
+    if (ignoring) {
+        /* We're only here to mark a variable as non-const */
+        if (in_constant_context()) {
+            DBufFree(&buf);
+            return OK;
+        }
+        var = FindVar(DBufValue(&buf), 0);
+        if (var) {
+            nonconst_debug(!var->is_constant, tr("Potential variable assignment considered non-constant because of context"));
+            var->is_constant = 0;
+        }
+        DBufFree(&buf);
+        return OK;
+    }
 
     /* Allow optional equals-sign:  SET var = value */
     ch = ParseNonSpaceChar(p, &r, 1);
@@ -624,6 +697,7 @@ int DoSet (Parser *p)
         return E_CANTNEST_FDEF;
     }
 
+    p->nonconst_expr = 0;
     r = EvaluateExpr(p, &v);
     if (r) {
         DBufFree(&buf);
@@ -638,7 +712,18 @@ int DoSet (Parser *p)
     }
     DBufFree(&buf2);
     if (*DBufValue(&buf) == '$') r = SetSysVar(DBufValue(&buf)+1, &v);
-    else r = SetVar(DBufValue(&buf), &v);
+    else {
+        if (p->nonconst_expr || !in_constant_context()) {
+            r = SetVar(DBufValue(&buf), &v, 1);
+        } else {
+            r = SetVar(DBufValue(&buf), &v, 0);
+        }
+        if (DebugFlag & DB_NONCONST) {
+            if (!in_constant_context() && !p->nonconst_expr) {
+                Wprint(tr("Variable assignment considered non-constant because of context"));
+            }
+        }
+    }
     if (buf.len > VAR_NAME_LEN) {
         Wprint(tr("Warning: Variable name `%.*s...' truncated to `%.*s'"),
                VAR_NAME_LEN, DBufValue(&buf), VAR_NAME_LEN, DBufValue(&buf));
@@ -692,17 +777,25 @@ int DoDump(ParsePtr p)
     int r;
     Var *v;
     DynamicBuffer buf;
+    int dump_constness = 0;
 
     if (PurgeMode) return OK;
 
     DBufInit(&buf);
     r = ParseToken(p, &buf);
     if (r) return r;
+    if (!strcmp(DBufValue(&buf), "-c")) {
+        dump_constness = 1;
+        DBufFree(&buf);
+        DBufInit(&buf);
+        r = ParseToken(p, &buf);
+        if (r) return r;
+    }
     if (!*DBufValue(&buf) ||
         *DBufValue(&buf) == '#' ||
         *DBufValue(&buf) == ';') {
         DBufFree(&buf);
-        DumpVarTable();
+        DumpVarTable(dump_constness);
         return OK;
     }
     fprintf(ErrFp, "%s  %s\n\n", VARIABLE, VALUE);
@@ -717,6 +810,11 @@ int DoDump(ParsePtr p)
             else {
                 fprintf(ErrFp, "%s  ", v->name);
                 PrintValue(&(v->v), ErrFp);
+                if (dump_constness) {
+                    if (v->is_constant) {
+                        fprintf(ErrFp, " <const>");
+                    }
+                }
                 fprintf(ErrFp, "\n");
             }
         }
@@ -738,7 +836,7 @@ int DoDump(ParsePtr p)
 /*  Dump the variable table to ErrFp.                         */
 /*                                                             */
 /***************************************************************/
-void DumpVarTable(void)
+void DumpVarTable(int dump_constness)
 {
     Var *v;
 
@@ -747,10 +845,32 @@ void DumpVarTable(void)
     hash_table_for_each(v, &VHashTbl) {
         fprintf(ErrFp, "%s  ", v->name);
         PrintValue(&(v->v), ErrFp);
+        if (dump_constness) {
+            if (v->is_constant) {
+                fprintf(ErrFp, " <const>");
+            }
+        }
         fprintf(ErrFp, "\n");
     }
 }
 
+void DumpUnusedVars(void)
+{
+    Var *v;
+    int done_header = 0;
+
+    hash_table_for_each(v, &VHashTbl) {
+        if (v->used_since_set) {
+            continue;
+        }
+        if (!done_header) {
+            fprintf(ErrFp, "%s\n", tr("The following variables were set, but not subsequently used:"));
+            done_header = 1;
+        }
+        fprintf(ErrFp, "\t%s - %s %s:%d\n", v->name, tr("defined at"), v->filename, v->lineno);
+    }
+}
+
 /***************************************************************/
 /*                                                             */
 /*  DestroyVars                                                */
@@ -794,6 +914,8 @@ int PreserveVar(char const *name)
     v = FindVar(name, 1);
     if (!v) return E_NO_MEM;
     v->preserve = 1;
+    /* Assume we're gonna use the variable */
+    v->used_since_set = 1;
     return OK;
 }
 
@@ -809,11 +931,10 @@ int DoPreserve (Parser *p)
     DynamicBuffer buf;
     DBufInit(&buf);
 
-    r = ParseToken(p, &buf);
-    if (r) return r;
-    if (!DBufLen(&buf)) {
+    r = ParseIdentifier(p, &buf);
+    if (r) {
         DBufFree(&buf);
-        return E_EOLN;
+        return r;
     }
 
     r = PreserveVar(DBufValue(&buf));
@@ -822,12 +943,14 @@ int DoPreserve (Parser *p)
 
 /* Keep going... */
     while(1) {
-        r = ParseToken(p, &buf);
-        if (r) return r;
-        if (!DBufLen(&buf)) {
+        r = ParseIdentifier(p, &buf);
+        if (r == E_EOLN) {
             DBufFree(&buf);
             return OK;
         }
+        if (r) {
+            return r;
+        }
         r = PreserveVar(DBufValue(&buf));
         DBufFree(&buf);
         if (r) return r;
@@ -946,6 +1069,7 @@ static SysVar SysVarArr[] = {
     {"SuppressLRM",    1,  INT_TYPE,     &SuppressLRM,         0,      1 },
     {"SysInclude",     0,  STR_TYPE,     &SysDir,              0,      0 },
     {"T",              0,  SPECIAL_TYPE, trig_date_func,       0,      0 },
+    {"Tb",             0,  SPECIAL_TYPE, trig_base_func,       0,      0 },
     {"Td",             0,  SPECIAL_TYPE, trig_day_func,        0,      0 },
     {"TerminalBackground", 0, SPECIAL_TYPE,  terminal_bg_func, 0,      0 },
     {"Thursday",       1,  TRANS_TYPE,   "Thursday",           0,      0 },
@@ -955,6 +1079,7 @@ static SysVar SysVarArr[] = {
     {"Today",          1,  TRANS_TYPE,   "today",              0,      0 },
     {"Tomorrow",       1,  TRANS_TYPE,   "tomorrow",           0,      0 },
     {"Tt",             0,  SPECIAL_TYPE, trig_time_func,       0,      0 },
+    {"Tu",             0,  SPECIAL_TYPE, trig_until_func,      0,      0 },
     {"Tuesday",        1,  TRANS_TYPE,   "Tuesday",            0,      0 },
     {"Tw",             0,  SPECIAL_TYPE, trig_wday_func,       0,      0 },
     {"Ty",             0,  SPECIAL_TYPE, trig_year_func,       0,      0 },
